==> nmrium-0.50.1.tgz: package/lib/data/data2d/Spectrum2D/getSlice.js <==
import { zoneToX } from 'ml-spectra-processing';
import { initiateDatum1D } from '../../data1d/Spectrum1D';
const BASE_INFO = {
    isFid: false,
    isComplex: false, // if isComplex is true that mean it contains real/ imaginary  x set, if not hid re/im button .
    dimension: 1,
};
export function getSlice(spectrum, position, options = {}) {
    const { sliceType = 'real' } = options;
    const { data: spectraData, info } = spectrum;
    const real = getRealData(spectraData, info);
    const imaginary = getImaginaryData(spectraData, info);
    const xLength = real.z[0].length;
    const yLength = real.z.length;
    const xStep = (real.maxX - real.minX) / (xLength - 1);
    const yStep = (real.maxY - real.minY) / (yLength - 1);
    const xIndex = Math.floor((position.x - real.minX) / xStep);
    const yIndex = Math.floor((position.y - real.minY) / yStep);
    if (xIndex < 0 || xIndex >= real.z[0].length)
        return;
    if (yIndex < 0 || yIndex >= real.z.length)
        return;
    const infoX = {
        nucleus: info.nucleus[0], // 1H, 13C, 19F, ...
        ...BASE_INFO,
    };
    const infoY = {
        nucleus: info.nucleus[1], // 1H, 13C, 19F, ...
        ...BASE_INFO,
    };
    const dataX = initiateData(real.minX, real.maxX, xLength);
    const dataY = initiateData(real.minY, real.maxY, yLength);
    if (['real', 'both'].includes(sliceType)) {
        for (let i = 0; i < xLength; i++) {
            dataX.re[i] += real.z[yIndex][i];
        }
        for (let i = 0; i < yLength; i++) {
            dataY.re[i] += real.z[i][xIndex];
        }
    }
    if (imaginary && ['imaginary', 'both'].includes(sliceType)) {
        infoX.isComplex = true;
        dataX.im = new Float64Array(xLength);
        for (let i = 0; i < xLength; i++) {
            dataX.im[i] += imaginary.z[yIndex][i];
        }
        //FT spectra vertical slicing should use ri instead of ir
        const imaginaryVerticalData = getImaginaryData(spectraData, info, {
            ftObjectKey: 'ri',
        });
        if (imaginaryVerticalData) {
            infoY.isComplex = true;
            for (let i = 0; i < yLength; i++) {
                dataY.im = new Float64Array(yLength);
                dataY.im[i] += imaginaryVerticalData.z[i][xIndex];
            }
        }
    }
    const horizontal = initiateDatum1D({ info: infoX, data: dataX });
    const vertical = initiateDatum1D({ info: infoY, data: dataY });
    return { horizontal, vertical };
}
function initiateData(from, to, length) {
    return {
        x: zoneToX({ from, to }, length),
        re: new Float64Array(length),
    };
}
function getRealData(data, info) {
    if (info.isFid) {
        return data.re;
    }
    return data.rr;
}
function getImaginaryData(data, info, options = {}) {
    if (info.isFid) {
        return data.im;
    }
    const { ftObjectKey = 'ir' } = options;
    return data?.[ftObjectKey];
}

==> nmrium-0.51.0.tgz: package/lib/data/data2d/Spectrum2D/getSlice.js <==
import { xSequentialFillFromTo } from 'ml-spectra-processing';
import { initiateDatum1D } from '../../data1d/Spectrum1D';
const BASE_INFO = {
    isFid: false,
    isComplex: false, // if isComplex is true that mean it contains real/ imaginary  x set, if not hid re/im button .
    dimension: 1,
};
export function getSlice(spectrum, position, options = {}) {
    const { sliceType = 'real' } = options;
    const { data: spectraData, info } = spectrum;
    const real = getRealData(spectraData, info);
    const imaginary = getImaginaryData(spectraData, info);
    const xLength = real.z[0].length;
    const yLength = real.z.length;
    const xStep = (real.maxX - real.minX) / (xLength - 1);
    const yStep = (real.maxY - real.minY) / (yLength - 1);
    const xIndex = Math.floor((position.x - real.minX) / xStep);
    const yIndex = Math.floor((position.y - real.minY) / yStep);
    if (xIndex < 0 || xIndex >= real.z[0].length)
        return;
    if (yIndex < 0 || yIndex >= real.z.length)
        return;
    const infoX = {
        nucleus: info.nucleus[0], // 1H, 13C, 19F, ...
        ...BASE_INFO,
    };
    const infoY = {
        nucleus: info.nucleus[1], // 1H, 13C, 19F, ...
        ...BASE_INFO,
    };
    const dataX = initiateData(real.minX, real.maxX, xLength);
    const dataY = initiateData(real.minY, real.maxY, yLength);
    if (['real', 'both'].includes(sliceType)) {
        for (let i = 0; i < xLength; i++) {
            dataX.re[i] += real.z[yIndex][i];
        }
        for (let i = 0; i < yLength; i++) {
            dataY.re[i] += real.z[i][xIndex];
        }
    }
    if (imaginary && ['imaginary', 'both'].includes(sliceType)) {
        infoX.isComplex = true;
        dataX.im = new Float64Array(xLength);
        for (let i = 0; i < xLength; i++) {
            dataX.im[i] += imaginary.z[yIndex][i];
        }
        //FT spectra vertical slicing should use ri instead of ir
        const imaginaryVerticalData = getImaginaryData(spectraData, info, {
            ftObjectKey: 'ri',
        });
        if (imaginaryVerticalData) {
            infoY.isComplex = true;
            for (let i = 0; i < yLength; i++) {
                dataY.im = new Float64Array(yLength);
                dataY.im[i] += imaginaryVerticalData.z[i][xIndex];
            }
        }
    }
    const horizontal = initiateDatum1D({ info: infoX, data: dataX });
    const vertical = initiateDatum1D({ info: infoY, data: dataY });
    return { horizontal, vertical };
}
function initiateData(from, to, size) {
    return {
        x: xSequentialFillFromTo({ from, to, size }),
        re: new Float64Array(length),
    };
}
function getRealData(data, info) {
    if (info.isFid) {
        return data.re;
    }
    return data.rr;
}
function getImaginaryData(data, info, options = {}) {
    if (info.isFid) {
        return data.im;
    }
    const { ftObjectKey = 'ir' } = options;
    return data?.[ftObjectKey];
}
